{--------------------------------------------------------------------------------------------------}
{
        Component :     TSCSwitchToggle
        Author    :     Sebastien Costa
        Date      :     10/11/2002

        Version   :     1.0

        Home Page :     http://web.tiscali.it/sebastien/delphi
        E-Mail    :     sebastien@tiscalinet.it

  Send me an E-mail if you have any comments, questions or bug report.
  If you find this component useful, I would enjoy hearing about it!
  Send me money for beer!!! Thank you!!!
  
  Legal issues:
  This component is freeware. I will not responsible for any damage
  that occurs when you use this component. Although it has been
  tested and works fine. USE IT AT YOUR OWN RISK !
  Feels free to use, modify, and redistribute it as long as this notice
  didn't removed or altered.

}
{--------------------------------------------------------------------------------------------------}
unit SCSwitchToggle;

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, Graphics, ExtCtrls;

type
  TSCState        = (stOn, stOff);
  TSCOrientation  = (orVertical, orHorizontal);
  TSCBorderStyle  = (bsNone, bsLowered, bsRaised);
  TSCBorderWidth  = (bwThin, bwLarge);
  TSCMouseControl = (mcClickAnyWhere, mcClickIndicator);
  TSCChangeEvent  = procedure(Sender : TObject; NewState: TSCState) of object;

  {-------------------------------------}
  TSCCustomSwitchToggle = class(TCustomControl)
  private
    { Private declarations }
    FGlyph        : TBitmap;
    FState        : TSCState;
    FDragToToggle : boolean;
    FMouseControl : TSCMouseControl;

    FOrientation  : TSCOrientation;

    FBorderStyle  : TSCBorderStyle;
    FBorderWidth  : TSCBorderWidth;
    FBorder       : Integer;

    FStretch      : boolean;

    FOnChange     : TSCChangeEvent;

    isDragging    : boolean;
    clickPoint    : TPoint;
    indicator     : TRect;

    procedure SetGlyph(const Value: TBitmap);
    procedure SetBorderStyle(const Value: TSCBorderStyle);
    procedure SetBorderWidth(const Value: TSCBorderWidth);
    procedure SetState(const Value: TSCState);
    procedure SetOrientation(const Value: TSCOrientation);
    procedure SetMouseControl(const Value: TSCMouseControl);
    procedure SetStretch(const Value: boolean);
    
  protected
    { Protected declarations }
    procedure Paint; override;
    procedure Click; override;
    procedure KeyPress(var Key: Char); override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure CMMouseLeave(var AMsg: TMessage);message CM_MOUSELEAVE;
    procedure WMSetFocus(var Message: TWMSetFocus); message WM_SETFOCUS;
    procedure WMKillFocus(var Message: TWMKillFocus); message WM_KILLFOCUS;

    property Glyph        : TBitmap  read FGlyph        write SetGlyph;
    property DragToToggle : boolean  read FDragToToggle write FDragToToggle default false;
    property MouseControl : TSCMouseControl read FMouseControl write SetMouseControl default mcClickAnyWhere;
    property State        : TSCState        read FState        write SetState        default stOn;
    property Orientation  : TSCOrientation  read FOrientation  write SetOrientation  default orVertical;
    property BorderStyle  : TSCBorderStyle  read FBorderStyle  write SetBorderStyle  default bsLowered;
    property BorderWidth  : TSCBorderWidth  read FBorderWidth  write SetBorderWidth  default bwThin;
    property Stretch      : boolean         read FStretch      write SetStretch      default false;
    property OnChange     : TSCChangeEvent  read FOnChange     write FOnChange;

  public
    { Public declarations }
    constructor Create( AOwner: TComponent ); override;
    destructor Destroy; override;
    procedure Toggle; virtual;

  published
    { Published declarations }
  end;

  {-------------------------------------}
  TSCSwitchToggle = class(TSCCustomSwitchToggle)
  private
    { Private declarations }
  protected
    { Protected declarations }
  public
    { Public declarations }
  published
    { Published declarations }
    property Glyph;
    property DragToToggle;
    property MouseControl;
    property State;
    property BorderStyle;
    property BorderWidth;
    property Orientation;
    property OnChange;
    property TabStop default true;
    property TabOrder;
    property Visible;
    property Color;
    property Stretch;

  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Seb', [TSCSwitchToggle]);
end;

{--------------------------------------------------------------------------------------------------}
constructor TSCCustomSwitchToggle.Create;
begin
  inherited;
  FGlyph        := TBitmap.Create;
  FDragToToggle := false;
  FState        := stOn;
  FBorderWidth  := bwThin;
  FBorderStyle  := bsLowered;
  FBorder       := 2;
  FOrientation  := orVertical;
  FMouseControl := mcClickAnyWhere;
  FStretch      := false;

  isDragging    := true;

  ControlStyle  := ControlStyle + [csFramed, csOpaque]; { reduce flicker on component redraw }
end;

{--------------------------------------------------------------------------------------------------}
destructor TSCCustomSwitchToggle.Destroy;
begin
  FGlyph.Free;
  inherited;
end;

{--------------------------------------------------------------------------------------------------}
procedure TSCCustomSwitchToggle.Paint;
var
  tmprect  : TRect;
  destrect : TRect;
begin
  if (Not Visible) And (Not (csDesigning in ComponentState)) then
    exit;

  inherited;

  canvas.Brush.Color := Color;
  canvas.Brush.Style := bsSolid;
  canvas.FillRect(ClientRect);

  tmprect := Rect(0,0,width,height);
  { draw border }
  if (FBorderStyle <> bsNone) then begin
    if (FBorderStyle = bsRaised)
      then
        Frame3D(canvas, tmprect, clWhite, clGray,  FBorder)
      else
        Frame3D(canvas, tmprect, clGray,  clWhite, FBorder);
  end;

  if Forientation = orVertical then
  begin
    destrect := Rect(FBorder,FBorder,Width-FBorder,(Height DIV 2)-FBorder);

    if FState = stOff then
      OffsetRect(destrect, 0, (Height DIV 2));
  end
  else
  begin
    destrect := Rect(FBorder,FBorder,(Width DIV 2)-FBorder,Height-FBorder);

    if FState = stOff then
      OffsetRect(destrect, (Width DIV 2), 0);
  end;

  if FStretch then
  begin
    indicator := destrect;
    Canvas.StretchDraw(indicator, FGlyph);
  end
  else
  begin
    indicator.left := (destrect.right  - destrect.left - FGlyph.Width ) DIV 2 + destrect.left;
    indicator.top  := (destrect.bottom - destrect.top  - FGlyph.Height) DIV 2 + destrect.top;
    indicator.right  := indicator.left + FGlyph.Width;
    indicator.bottom := indicator.top  + FGlyph.Height;
    Canvas.Draw(indicator.left, indicator.top, FGlyph);
  end;

  if focused then canvas.DrawFocusRect(ClientRect);
end;

{--------------------------------------------------------------------------------------------------}
procedure TSCCustomSwitchToggle.SetGlyph(const Value: TBitmap);
begin
  FGlyph.Assign(Value);
  Repaint;
end;

{--------------------------------------------------------------------------------------------------}
procedure TSCCustomSwitchToggle.SetState(const Value: TSCState);
begin
  if (FState <> Value) then
  begin
    FState := Value;
    Repaint;
  end;
end;

{--------------------------------------------------------------------------------------------------}
procedure TSCCustomSwitchToggle.SetStretch(const Value: boolean);
begin
  if (FStretch <> Value) then
  begin
    FStretch := Value;
    Repaint;
  end;
end;

{--------------------------------------------------------------------------------------------------}
procedure TSCCustomSwitchToggle.SetMouseControl(const Value: TSCMouseControl);
begin
  if (FMouseControl <> Value) then
  begin
    FMouseControl := Value;
    Repaint;
  end;
end;

{--------------------------------------------------------------------------------------------------}
procedure TSCCustomSwitchToggle.SetOrientation(const Value: TSCOrientation);
begin
  if (FOrientation <> Value) then
  begin
    FOrientation := Value;
    Repaint;
  end;
end;

{--------------------------------------------------------------------------------------------------}
procedure TSCCustomSwitchToggle.SetBorderStyle(const Value: TSCBorderStyle);
begin
  if (FBorderStyle <> Value) then
  begin
    FBorderStyle := Value;
    if (FBorderStyle = bsNone) then
      FBorder := 0
    else
    begin
      if (FBorderWidth = bwThin)
        then FBorder := 1
        else FBorder := 2;
    end;
    Repaint;
  end;
end;

{--------------------------------------------------------------------------------------------------}
procedure TSCCustomSwitchToggle.SetBorderWidth(const Value: TSCBorderWidth);
begin
  if (FBorderWidth <> Value) then
  begin
    FBorderWidth := Value;

    if (FBorderWidth = bwThin)
      then FBorder := 1
      else FBorder := 2;

    Repaint;
  end;
end;

{--------------------------------------------------------------------------------------------------}
procedure TSCCustomSwitchToggle.MouseDown(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
Begin
  clickPoint.x := x;
  clickPoint.y := y;
  if FDragToToggle then
  begin
    if (Button = mbleft) and enabled then
    begin
      SetFocus;
      if not ((FMouseControl = mcClickIndicator) and (not PtInRect(indicator, clickPoint))) then
      begin
        isDragging := true;
      end;
    end;
  end;
  inherited MouseDown(Button, Shift, X, Y);
end;

{--------------------------------------------------------------------------------------------------}
procedure TSCCustomSwitchToggle.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if isDragging then
  begin
    isDragging := false;
    if FOrientation = orVertical then
    begin
      if ((FState = stOn)  and (Y > clickPoint.Y)) or
         ((FState = stOff) and (Y < clickPoint.Y)) then
        Toggle;
    end
    else
    begin
      if ((FState = stOn)  and (X > clickPoint.X)) or
         ((FState = stOff) and (X < clickPoint.X)) then
        Toggle;
    end;
  end;
  inherited MouseUp(Button, Shift, X, Y);
end;

{--------------------------------------------------------------------------------------------------}
procedure TSCCustomSwitchToggle.Click;
begin
  if Not FDragToToggle Then
  begin
    SetFocus;
    if not ((FMouseControl = mcClickIndicator) and (not PtInRect(indicator, clickPoint))) then
      Toggle;
    inherited Click;
  end;
end;

{--------------------------------------------------------------------------------------------------}
procedure TSCCustomSwitchToggle.KeyPress(var Key: Char);
begin
  inherited KeyPress(Key);
  if Key = ' ' then
  begin
    Toggle;
    Repaint;
  end;
end;

{--------------------------------------------------------------------------------------------------}
procedure TSCCustomSwitchToggle.CMMouseLeave(var AMsg: TMessage);
begin
  ReleaseCapture;
  isDragging := false;
end;

{--------------------------------------------------------------------------------------------------}
procedure TSCCustomSwitchToggle.WMSetFocus(var Message: TWMSetFocus);
begin
  inherited;
  Repaint;
end;

{--------------------------------------------------------------------------------------------------}
procedure TSCCustomSwitchToggle.WMKillFocus(var Message: TWMKillFocus);
begin
  inherited;
  Repaint;
end;

{--------------------------------------------------------------------------------------------------}
procedure TSCCustomSwitchToggle.Toggle;
begin
  if FState = stOn then FState := stOff else FState := stOn;
  Repaint;
  if Assigned(FOnChange) then FOnChange(Self, FState);
end;

end.
